@@ -299,8 +299,10 @@ size_t encoder_write_packet(struct encoder_op *op, struct encoder_op_packet *pac
 void encoder_write_packet_all(struct encoder *encoder, struct encoder_op_packet *packet)
    {
    struct encoder_op *iter;
+   struct timespec ms10 = { 0, 10000000 };
    
-   pthread_mutex_lock(&encoder->mutex);
+   while (pthread_mutex_trylock(&encoder->mutex))
+      nanosleep(&ms10, NULL);
    for (iter = encoder->output_chain; iter; iter = iter->next)
       encoder_write_packet(iter, packet);
    pthread_mutex_unlock(&encoder->mutex);
@@ -375,7 +377,8 @@ int encoder_client_set_flush(struct encoder_op *op)
 struct encoder_op *encoder_register_client(struct threads_info *ti, int numeric_id)
    {
    struct encoder *enc;
-   struct encoder_op *op, *iter;
+   struct encoder_op *op;
+   struct timespec ms10 = { 0, 10000000 };
    
    if (numeric_id >= ti->n_encoders || numeric_id < 0)
       {
@@ -396,26 +399,24 @@ struct encoder_op *encoder_register_client(struct threads_info *ti, int numeric_
    enc = ti->encoder[numeric_id];
    op->encoder = enc;
    pthread_mutex_init(&op->mutex, NULL);
-   if ((iter = enc->output_chain))/* insert at end of list so we don't have to mutex lock */
-      {
-      while(iter->next != NULL)
-         iter = iter->next;
-      iter->next = op;
-      }
-   else
-      enc->output_chain = op;
+   while (pthread_mutex_trylock(&op->encoder->mutex))
+      nanosleep(&ms10, NULL);
+   op->next = enc->output_chain;
+   enc->output_chain = op;
    enc->client_count++;
+   pthread_mutex_unlock(&op->encoder->mutex);
    return op;
    }
    
 void encoder_unregister_client(struct encoder_op *op)
    {
    struct encoder_op *iter;
+   struct timespec ms10 = { 0, 10000000 };	/* ten milliseconds */
    
    fprintf(stderr, "encoder_unregister_client called\n");
-   pthread_mutex_lock(&op->encoder->mutex);
-   iter = op->encoder->output_chain;
-   if (iter == op)
+   while (pthread_mutex_trylock(&op->encoder->mutex))
+      nanosleep(&ms10, NULL);
+   if ((iter = op->encoder->output_chain) == op)
       op->encoder->output_chain = op->next;
    else
       {
@@ -423,10 +424,10 @@ void encoder_unregister_client(struct encoder_op *op)
          iter = iter->next;
       iter->next = op->next;
       }
+   op->encoder->client_count--;
    pthread_mutex_unlock(&op->encoder->mutex);
    pthread_mutex_destroy(&op->mutex);
    jack_ringbuffer_free(op->packet_rb);
-   op->encoder->client_count--;
    free(op);
    fprintf(stderr, "encoder_unregister_client finished\n");
    }
@@ -436,7 +437,6 @@ void *encoder_main(void *args)
    struct encoder *self = args;
    struct timespec ms10 = { 0, 10000000 };	/* ten milliseconds */
 
-   fprintf(stderr, "encoder_main: got numeric id %d\n", self->numeric_id);
    while(!self->thread_terminate_f)
       {
       pthread_mutex_lock(&self->flush_mutex);
@@ -455,7 +455,6 @@ void *encoder_main(void *args)
       pthread_mutex_unlock(&self->flush_mutex);
       nanosleep(&ms10, NULL);
       }
-   fprintf(stderr, "encoder_main: got the signal to terminate; acting on it\n");
    return NULL;
    }
 
@@ -540,7 +539,10 @@ int encoder_start(struct threads_info *ti, struct universal_vars *uv, void *othe
       while (self->encoder_state == ES_STARTING)
          nanosleep(&ms10, NULL);
       if (self->encoder_state == ES_STOPPED)
+         {
+         fprintf(stderr, "encoder_start: encoder failed during initialisation\n");
          goto failed;
+         }
       fprintf(stderr, "encoder_start: successfully started the encoder\n");
       return SUCCEEDED;
       }
@@ -611,10 +613,22 @@ int encoder_new_metadata(struct threads_info *ti, struct universal_vars *uv, voi
          free(self->artist_title);
       if (self->artist_title_mp3)
          free(self->artist_title_mp3);
-      self->artist = strdup(ev->artist);
-      self->title = strdup(ev->title);
-      self->artist_title = strdup(ev->artist_title);
-      self->artist_title_mp3 = strdup(ev->artist_title_mp3);
+      if (ev->artist)
+         self->artist = strdup(ev->artist);
+      else
+         self->artist = strdup("");
+      if (ev->title)
+         self->title = strdup(ev->title);
+      else
+         self->title = strdup("");
+      if (ev->artist_title)
+         self->artist_title = strdup(ev->artist_title);
+      else
+         self->artist_title = strdup("");
+      if (ev->artist_title_mp3)
+         self->artist_title_mp3 = strdup(ev->artist_title_mp3);
+      else
+         self->artist_title_mp3 = strdup("");
       if (!(self->artist && self->title && self->artist_title && self->artist_title_mp3))
          {
          pthread_mutex_unlock(&self->metadata_mutex);
@@ -633,12 +647,20 @@ int encoder_new_metaformat(struct threads_info *ti, struct universal_vars *uv, v
    struct encoder *self = ti->encoder[uv->tab];
    struct encoder_vars *ev = other;
 
-   self->new_metadata = FALSE;
    pthread_mutex_lock(&self->metadata_mutex);
+   self->new_metadata = FALSE;
+   if (self->metaformat)
+      free(self->metaformat);
+   if (self->metaformat_mp3)
+      free(self->metaformat_mp3);
    self->metaformat = ev->metaformat;
    ev->metaformat = NULL;
    self->metaformat_mp3 = ev->metaformat_mp3;
    ev->metaformat_mp3 = NULL;
+   if (!self->metaformat)
+      self->metaformat = strdup("");
+   if (!self->metaformat_mp3)
+      self->metaformat_mp3 = strdup("");
    self->new_metadata = TRUE;
    pthread_mutex_unlock(&self->metadata_mutex);
    return SUCCEEDED;
@@ -663,8 +685,13 @@ struct encoder *encoder_init(struct threads_info *ti, int numeric_id)
       }
    self->threads_info = ti;
    self->numeric_id = numeric_id;
+   self->artist = strdup("");
+   self->title = strdup("");
+   self->artist_title = strdup("");
+   self->artist_title_mp3 = strdup("");
    self->metaformat = strdup("%s");
    self->metaformat_mp3 = strdup("%s");
+   while ((self->oggserial = rand()) + 20000 < 0 || self->oggserial < 100);
    pthread_mutex_init(&self->mutex, NULL);
    pthread_mutex_init(&self->metadata_mutex, NULL);
    pthread_mutex_init(&self->flush_mutex, NULL);
@@ -673,19 +700,25 @@ struct encoder *encoder_init(struct threads_info *ti, int numeric_id)
       fprintf(stderr, "encoder_init: pthread_create call failed\n");
       return NULL;
       }
-   fprintf(stderr, "encoder_init: created encoder %d\n", numeric_id);
    /* the input ringbuffer will be allocated when the encoder is started */
    return self;
    }
 
 void encoder_destroy(struct encoder *self)
    {
-   fprintf(stderr, "encoder_destroy: freeing the resources of the encoder\n");
    self->thread_terminate_f = TRUE;
    pthread_join(self->thread_h, NULL);
    pthread_mutex_destroy(&self->mutex);
    pthread_mutex_destroy(&self->metadata_mutex);
    pthread_mutex_destroy(&self->flush_mutex);
+   if (self->rs_input[0])
+      free(self->rs_input[0]);
+   if (self->rs_input[1])
+      free(self->rs_input[1]);
+   if (self->metaformat)
+      free(self->metaformat);
+   if (self->metaformat_mp3)
+      free(self->metaformat_mp3);
    if (self->input_rb[0])
       jack_ringbuffer_free(self->input_rb[0]);
    if (self->input_rb[1])
