@@ -98,6 +98,8 @@ int shutdown;
 int jack_closed_f;
 /* flag to indicate whether to use the player reading function which supports speed variance */
 int speed_variance;
+/* flag to indicate if microphone audio should be present in the dj stream */
+int mic_to_dj;
 /* buffers that process_audio uses when reading media player data */
 sample_t *lp_lc, *lp_rc, *rp_lc, *rp_rc, *jp_lc, *jp_rc, *ip_lc, *ip_rc;
 sample_t *lp_lcf, *lp_rcf, *rp_lcf, *rp_rcf, *jp_lcf, *jp_rcf, *ip_lcf, *ip_rcf;
@@ -124,6 +126,9 @@ int reset_vu_stats_f;			/* when set the mixer will reset the above */
 float *dblookup, *antidblookup;		/* a table for speeding up log / antilog operations */
 float *fade_table;			/* a table of values used for autofade */
 float dfmod;				/* used to reduce the ducking factor */
+float dj_audio_level;			/* used to reduce the level of dj audio */
+float dj_audio_gain = 1.0;		/* same as above but not in dB */
+float current_dj_audio_level = 0.0;
 
 struct compressor left_mic_compressor =
    {
@@ -176,6 +181,7 @@ sample_t lp_lc_strf = 1.0, lp_rc_strf = 1.0, rp_lc_strf = 1.0, rp_rc_strf = 1.0;
 sample_t jp_lc_strf = 1.0, jp_rc_strf = 1.0, jp_lc_audf = 1.0, jp_rc_audf = 1.0;
 sample_t ip_lc_strf = 1.0, ip_rc_strf = 1.0, ip_lc_audf = 0.0, ip_rc_audf = 0.0;
 sample_t mic_lc = 0.0, mic_rc = 0.0;	/* microphone gain */
+sample_t dj_mic_aud;		/* additional gain applied to mic in dj audio */
 	 
 sample_t mic_linv = 1.0, mic_rinv = 1.0; 	/* For phase inversion when set to -1.0 */
 sample_t mic_lgain = 1.0, mic_rgain = 1.0;	/* User specified mic attenuation */
@@ -334,9 +340,15 @@ void update_smoothed_volumes()
    float vol;
    float xprop, yprop;
    const float bias = 0.35386F;
-   
+
    timeout++;
    
+   if (dj_audio_level != current_dj_audio_level)
+      {
+      current_dj_audio_level = dj_audio_level;
+      dj_audio_gain = db2level(dj_audio_level);
+      }
+   
    if (crossfade != current_crossfade)
       {
       if (crossfade > current_crossfade)
@@ -451,6 +463,8 @@ void update_smoothed_volumes()
    else
       vol = current_volume * 0.06666666F;
    dfmod = 1.0F / (vol * vol + 1.0F);
+
+   dj_mic_aud = (float)mic_to_dj;
    }
 
 /* update_mic_and_aux: provide gradual mute/unmute for mic and aux buttons */
@@ -851,9 +865,9 @@ void process_audio(jack_nframes_t nframes)
 	 
          if (stream_monitor == FALSE)
 	    {
-	    *lap = ((*lplcp * lp_lc_aud) + (*rplcp * rp_lc_aud) + (*jplcp * jp_lc_aud)) * df + lc_micmix + (*iplcp * ip_lc_aud) + (ip_lc_fade * ip_lc_aud) +
+	    *lap = ((*lplcp * lp_lc_aud) + (*rplcp * rp_lc_aud) + (*jplcp * jp_lc_aud)) * df + (lc_micmix * dj_mic_aud) + (*iplcp * ip_lc_aud) + (ip_lc_fade * ip_lc_aud) +
             (lp_lc_fade * lp_lc_audf) + (rp_lc_fade * rp_lc_audf) + (jp_lc_fade * jp_lc_audf);
-	    *rap = ((*lprcp * lp_rc_aud) + (*rprcp * rp_rc_aud) + (*jprcp * jp_rc_aud)) * df + rc_micmix + (*iprcp * ip_rc_aud) + (ip_rc_fade * ip_rc_aud) +
+	    *rap = ((*lprcp * lp_rc_aud) + (*rprcp * rp_rc_aud) + (*jprcp * jp_rc_aud)) * df + (rc_micmix * dj_mic_aud) + (*iprcp * ip_rc_aud) + (ip_rc_fade * ip_rc_aud) +
             (lp_rc_fade * lp_rc_audf) + (rp_rc_fade * rp_rc_audf) + (jp_rc_fade * jp_rc_audf);
 	    compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
 	    *lap *= compressor_gain;
@@ -882,6 +896,9 @@ void process_audio(jack_nframes_t nframes)
                }
             }
 	 	 
+	 *lap *= dj_audio_gain;
+	 *rap *= dj_audio_gain;	 
+	 	 
 	 /* make note of the peak volume levels */
 	 lmic = fabsf(lmic);
 	 if (lmic > 1.0F)
@@ -1002,9 +1019,6 @@ void process_audio(jack_nframes_t nframes)
             /* do the phone send mix */
             *lpsp = lc_micmix + (*jplcp * jp_lc_str) + (jp_lc_fade * jp_lc_strf);
             *rpsp = rc_micmix + (*jprcp * jp_rc_str) + (jp_rc_fade * jp_rc_strf);
-            compressor_gain = db2level(limiter(&phone_limiter, *lpsp, *rpsp));
-            *lpsp *= compressor_gain;
-            *rpsp *= compressor_gain;
             
             if (lmic_flux || rmic_flux || aux_flux)
                update_mic_and_aux();		/* smooth mic rise and fall mute/unmute */
@@ -1019,11 +1033,15 @@ void process_audio(jack_nframes_t nframes)
                right_peak = fabs(*rprcp);
 
             /* The main mix */
-            *lsp = ((*lplcp * lp_lc_str) + (*rplcp * rp_lc_str) + (*lxp * aux_lc)) + *lprp +
+            *lsp = ((*lplcp * lp_lc_str) + (*rplcp * rp_lc_str) + (*lxp * aux_lc)) + *lprp + *lpsp +
             (lp_lc_fade * lp_rc_strf) + (rp_lc_fade * rp_lc_strf) + (*iplcp * ip_lc_str) + (ip_lc_fade * ip_lc_strf);
-            *rsp = ((*lprcp * lp_rc_str) + (*rprcp * rp_rc_str) + (*rxp * aux_rc)) + *rprp +
+            *rsp = ((*lprcp * lp_rc_str) + (*rprcp * rp_rc_str) + (*rxp * aux_rc)) + *rprp + *rpsp +
             (lp_rc_fade * lp_rc_strf) + (rp_rc_fade * rp_rc_strf) + (*iprcp * ip_rc_str) + (ip_rc_fade * ip_rc_strf);
             
+            compressor_gain = db2level(limiter(&phone_limiter, *lpsp, *rpsp));
+            *lpsp *= compressor_gain;
+            *rpsp *= compressor_gain;
+
             /* apply normalization at the stream level */
             str_normalizer_gain = db2level(normalizer(&str_normalizer, *lsp, *rsp));
             
@@ -1039,9 +1057,9 @@ void process_audio(jack_nframes_t nframes)
             if (stream_monitor == FALSE)
                {
                *lap = ((*lplcp * lp_lc_aud) + (*rplcp * rp_lc_aud)) + *lprp +
-               (lp_lc_fade * lp_rc_audf) + (rp_lc_fade * rp_lc_audf) + (*iplcp * ip_lc_aud) + (ip_lc_fade * ip_lc_audf);
+               (lp_lc_fade * lp_rc_audf) + (rp_lc_fade * rp_lc_audf) + (*iplcp * ip_lc_aud) + (ip_lc_fade * ip_lc_audf) + (lc_micmix * dj_mic_aud) + (*jplcp * jp_lc_str) + (jp_lc_fade * jp_lc_strf);
                *rap = ((*lprcp * lp_rc_aud) + (*rprcp * rp_rc_aud)) + *rprp +
-               (lp_rc_fade * lp_rc_audf) + (rp_rc_fade * rp_rc_audf) + (*iprcp * ip_rc_aud) + (ip_rc_fade * ip_rc_audf);
+               (lp_rc_fade * lp_rc_audf) + (rp_rc_fade * rp_rc_audf) + (*iprcp * ip_rc_aud) + (ip_rc_fade * ip_rc_audf) + (rc_micmix * dj_mic_aud) + (*jprcp * jp_rc_str) + (jp_rc_fade * jp_rc_strf);
                compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
                *lap *= compressor_gain;
                *rap *= compressor_gain;
@@ -1069,6 +1087,9 @@ void process_audio(jack_nframes_t nframes)
                   }
                }
                   
+            *lap *= dj_audio_gain;
+            *rap *= dj_audio_gain;	 
+                  
             /* make note of the peak volume levels */
             lmic = fabsf(lmic);
             if (lmic > 1.0F)
@@ -1223,10 +1244,13 @@ void process_audio(jack_nframes_t nframes)
                *lpsp *= compressor_gain;
                *rpsp *= compressor_gain;
                
-               if (stream_monitor == FALSE)
+               if (stream_monitor == FALSE) /* the DJ can hear the VOIP phone call */
                   {
-                  *lap = *lprp;	/* the DJ can hear the VOIP phone call */
-                  *rap = *rprp;
+                  *lap = (*lsp * mb_lc_aud) + (*jplcp * jp_lc_aud) + (lc_micmix * dj_mic_aud) + (jp_lc_fade * jp_lc_strf) + *lprp;
+                  *rap = (*rsp * mb_lc_aud) + (*jprcp * jp_rc_aud) + (rc_micmix * dj_mic_aud) + (jp_rc_fade * jp_rc_strf) + *rprp;
+                  compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
+                  *lap *= compressor_gain;
+                  *rap *= compressor_gain;
                   }
                else
                   {
@@ -1251,6 +1275,9 @@ void process_audio(jack_nframes_t nframes)
                      }
                   }
                      
+               *lap *= dj_audio_gain;
+               *rap *= dj_audio_gain;	 
+                     
                /* make note of the peak volume levels */
                lmic = fabsf(lmic);
                if (lmic > 1.0F)
@@ -1410,9 +1437,9 @@ void process_audio(jack_nframes_t nframes)
                   
                   if (stream_monitor == FALSE)
                      {
-                     *lap = ((*lplcp * lp_lc_aud) + (*rplcp * rp_lc_aud) + (*jplcp * jp_lc_aud)) * df + lc_micmix + (*iplcp * ip_lc_aud) + (ip_lc_fade * ip_lc_audf) +
+                     *lap = ((*lplcp * lp_lc_aud) + (*rplcp * rp_lc_aud) + (*jplcp * jp_lc_aud)) * df + (lc_micmix * dj_mic_aud) + (*iplcp * ip_lc_aud) + (ip_lc_fade * ip_lc_audf) +
                      (lp_lc_fade * lp_lc_audf) + (rp_lc_fade * rp_lc_audf) + (jp_lc_fade * jp_lc_audf);
-                     *rap = ((*lprcp * lp_rc_aud) + (*rprcp * rp_rc_aud) + (*jprcp * jp_rc_aud)) * df + rc_micmix + (*iprcp * ip_rc_aud) + (ip_rc_fade * ip_rc_audf) +
+                     *rap = ((*lprcp * lp_rc_aud) + (*rprcp * rp_rc_aud) + (*jprcp * jp_rc_aud)) * df + (rc_micmix * dj_mic_aud) + (*iprcp * ip_rc_aud) + (ip_rc_fade * ip_rc_audf) +
                      (lp_rc_fade * lp_rc_audf) + (rp_rc_fade * rp_rc_audf) + (jp_rc_fade * jp_rc_audf);
                      compressor_gain = db2level(limiter(&audio_limiter, *lap, *rap));
                      *lap *= compressor_gain;
@@ -1441,6 +1468,9 @@ void process_audio(jack_nframes_t nframes)
                         }
                      }
                         
+                  *lap *= dj_audio_gain;
+                  *rap *= dj_audio_gain;	 
+                        
                   /* make note of the peak volume levels */
                   lmic = fabsf(lmic);
                   if (lmic > 1.0F)
@@ -1474,13 +1504,19 @@ void process_audio(jack_nframes_t nframes)
             else
                if (simple_mixer == TRUE)
                   {
-                  /* simple mixer code - no headroom needed */
-                  /* just a straight copy of left player audio data to the output ports */
-                  /* and no meters */
                   if (left_audio)
                      {
-                     memcpy(la_buffer, lp_lc, nframes * sizeof (sample_t));
-                     memcpy(ra_buffer, lp_rc, nframes * sizeof (sample_t));
+                     if (dj_audio_level != current_dj_audio_level)
+                        {
+                        current_dj_audio_level = dj_audio_level;
+                        dj_audio_gain = db2level(dj_audio_level);
+                        }
+                     samples_todo = nframes;
+                     while(samples_todo--)
+                        {
+                        *lap++ = *lplcp++ * dj_audio_gain;
+                        *rap++ = *lprcp++ * dj_audio_gain;
+                        }
                      }
                   else
                      {
@@ -1595,6 +1631,7 @@ void atexit_handler()
 void send_metadata_update(struct xlp_dynamic_metadata *dm)
    {
    pthread_mutex_lock(&(dm->meta_mutex));
+   fprintf(stderr, "dm->data_type = %d\n", dm->data_type);
    if (dm->data_type < DM_JOINED_UC)
       {
       fprintf(stdout, "new_metadata=d%d:%sd%d:%sd%d:%sd9:%09dd9:%09dx\n", strlen(dm->artist), dm->artist, strlen(dm->title), dm->title, strlen(dm->artist_title), dm->artist_title, dm->current_audio_context, dm->rbdelay);
@@ -1609,7 +1646,7 @@ void send_metadata_update(struct xlp_dynamic_metadata *dm)
 
 int main(int argc, char **argv)
    {
-   FILE *fp=stdin;
+   FILE *fp =stdin;
    char *client_name = 0;
    const char **inport;
    const char **outport;
@@ -1622,7 +1659,7 @@ int main(int argc, char **argv)
    int flush_left, flush_right, flush_jingles, flush_interlude;
    float comp_ducking, de_ess;
    int mic_left_invert, mic_right_invert, mic_left_level, mic_right_level,
-   					mic_left_balance, mic_right_balance;
+       mic_left_balance, mic_right_balance;
    int i, new_left_pause, new_right_pause;
    int ngthresh, nghyster, ngatten, ngontime, ngofftime;
    static int old_lmic_on = 0, old_rmic_on = 0, old_aux_on = 0;
@@ -1632,7 +1669,7 @@ int main(int argc, char **argv)
    int sync = FALSE;
    struct rms_calc *lm_fast_rms, *lm_slow_rms, *rm_fast_rms, *rm_slow_rms;
 
-   setenv("LC_ALL", "C", 1);			/* ensure proper sscanf operation */
+   setenv("LC_ALL", "C", 1);		/* ensure proper sscanf operation */
 
    if (sigfillset(&mixersigset))
       {
@@ -1648,7 +1685,8 @@ int main(int argc, char **argv)
    strcpy(client_name, "idjc-mx");
 
    atexit(atexit_handler);
-   if((client = jack_client_new(client_name)) == 0)
+
+   if((client = jack_client_open(client_name, JackUseExactName | JackServerName, NULL, getenv("IDJC_JACK_SERVER"))) == 0)
       {
       printf("IDJC: Error\n");
       fflush(stdout);
@@ -1783,8 +1821,14 @@ int main(int argc, char **argv)
       exit(5);
       }
    else
+      {
+      plr_l->fadeindex = FB_SIZE;
+      plr_r->fadeindex = FB_SIZE;
+      plr_j->fadeindex = FB_SIZE;
+      plr_i->fadeindex = FB_SIZE;
       for (i = 0; i < FB_SIZE; i ++)
          fade_table[i] = pow10f(i / -20000.0);
+      }
          
    nframes = jack_get_sample_rate (client);
    lp_lc = ialloc(nframes);
@@ -1835,10 +1879,10 @@ int main(int argc, char **argv)
  
    /* Make voip input audio available on the voip output so you can hear your own voice */
    /* and hear jingles and mixed back audio too */
-   /* If the voip app provides this then you can break the connection in the voip app */
-   /* launcher script */
-   jack_connect(client, "idjc-mx:voip_send_lt", "idjc-mx:voip_recv_lt");
-   jack_connect(client, "idjc-mx:voip_send_rt", "idjc-mx:voip_recv_rt");
+   /* If the voip app provides this you can break the connection automatically */
+   /* using the events feature in idjc prefs and the jack_disconnect program */
+   /*jack_connect(client, "idjc-mx:voip_send_lt", "idjc-mx:voip_recv_lt");
+   jack_connect(client, "idjc-mx:voip_send_rt", "idjc-mx:voip_recv_rt");*/
       
    alarm(3);		/* handles timeouts on media player worker threads */
       
@@ -1952,10 +1996,11 @@ int main(int argc, char **argv)
 
       if (!strcmp(action, "micmixstats"))
          {
-	 if (sscanf(microphone_string, ":%d:%d:%d:%d:%d:%d:%d:%f:", &mic_filter,
+	 if (sscanf(microphone_string, ":%d:%d:%d:%d:%d:%d:%d:%f:%d:", &mic_filter,
 	 	 &mic_left_invert, &mic_right_invert, &mic_left_level, &mic_right_level,
 		 &mic_left_balance, &mic_right_balance,
-		 &headroom_db) != 8)
+		 &headroom_db,
+		 &mic_to_dj) != 9)
 	    {
 	    fprintf(stderr, "mixer got bad microphone string\n");
 	    break;
@@ -2040,10 +2085,10 @@ int main(int argc, char **argv)
       if (!strcmp(action, "mixstats"))
          {
          if(sscanf(mixer_string,
-                ":%03d:%03d:%03d:%03d:%03d:%d:%1d%1d%1d%1d%1d:%1d%1d:%1d%1d%1d:%1d%1d%1d%1d:%1d:%1d:%1d:%1d:%1d:%f:%f:%1d:",
+                ":%03d:%03d:%03d:%03d:%03d:%d:%1d%1d%1d%1d%1d:%1d%1d:%1d%1d%1d:%1d%1d%1d%1d:%1d:%1d:%1d:%1d:%1d:%f:%f:%1d:%f:",
       		&volume, &crossfade, &jinglesvolume, &interludevol, &mixbackvol, &jingles_playing,
                 &left_stream, &left_audio, &right_stream, &right_audio, &stream_monitor,
-		&new_left_pause, &new_right_pause, &lmic_on, &rmic_on, &aux_on, &flush_left, &flush_right, &flush_jingles, &flush_interlude, &simple_mixer, &eot_alarm_f, &mixermode, &fadeout_f, &main_play, &(plr_l->newpbspeed), &(plr_r->newpbspeed), &speed_variance) !=28)
+		&new_left_pause, &new_right_pause, &lmic_on, &rmic_on, &aux_on, &flush_left, &flush_right, &flush_jingles, &flush_interlude, &simple_mixer, &eot_alarm_f, &mixermode, &fadeout_f, &main_play, &(plr_l->newpbspeed), &(plr_r->newpbspeed), &speed_variance, &dj_audio_level) !=29)
 	    {
 	    fprintf(stderr, "mixer got bad mixer string\n");
 	    break;
