@@ -84,3 +84,14 @@ int kvp_apply_to_dict(struct kvpdict *dp, char *key, char *target)
       }
    return 0;				/* No matches */
    }
+
+void kvp_free_dict(struct kvpdict *dp)
+   {
+   while (dp->key)
+      {
+      if (*(dp->target))
+         free(*(dp->target));
+      *dp->target = NULL;
+      dp++;
+      }
+   }
