@@ -80,6 +80,8 @@ static void mp3decode_eject(struct xlplayer *xlplayer)
    {
    struct mp3decode_vars *self = xlplayer->dec_data;
  
+   jack_ringbuffer_free(self->lrb);
+   jack_ringbuffer_free(self->rrb);
    if (self->resample)
       {
       if (xlplayer->src_data.data_in)
@@ -102,8 +104,8 @@ static void mp3decode_init(struct xlplayer *xlplayer)
    struct mp3decode_vars *self = xlplayer->dec_data;
    struct mp3taginfo *ti = &self->taginfo;
    off_t start, end, offset;
-   int retcode, dump, index;
-   int src_error, luv1, luv2;
+   int retcode, dump, index, rbsize;
+   int src_error, luv1, luv2, samples;
    float size, seek_pc, frac, scale;
     
    if (xlplayer->seek_s)
@@ -189,70 +191,140 @@ static void mp3decode_init(struct xlplayer *xlplayer)
          xlplayer->command = CMD_COMPLETE;
          }
       }
+   samples = self->synth.pcm.length;
+   rbsize = (samples * 2 + ti->end_frames_drop) * sizeof (float);
+   self->lrb = jack_ringbuffer_create(rbsize);
+   self->rrb = jack_ringbuffer_create(rbsize);
    }
-   
+
 static void mp3decode_play(struct xlplayer *xlplayer)
    {
    struct mp3decode_vars *self = xlplayer->dec_data;
+   struct mp3taginfo *ti = &self->taginfo;
    struct mad_pcm *pcm;
    jack_default_audio_sample_t *lp, *rp, *dp, gain;
-   mad_fixed_t const *left_ch, *right_ch;
-   int nchannels, nsamples, frame_code;
+   mad_fixed_t *left_ch, *right_ch, *lc, *rc;
+   int nchannels, nsamples, frame_code, pcmlength, toskip;
    SRC_DATA *src_data = &(xlplayer->src_data);
    struct chapter *chapter;
-   
+
    pcm = &(self->synth.pcm);
-   left_ch = pcm->samples[0];
+   left_ch = lc = pcm->samples[0];
    if ((nchannels = pcm->channels) == 2)
-      right_ch = pcm->samples[1];
+      right_ch = rc = pcm->samples[1];
    else
-      right_ch = NULL;
+      right_ch = rc = NULL;
+   pcmlength = pcm->length;
+
+   if (ti->end_frames_drop)
+      {
+      if (ti->start_frames_drop)	/* calculate initial frame droppage for gapless playback */
+         {
+         if (pcmlength <= ti->start_frames_drop)
+            {
+            toskip = pcmlength;
+            pcmlength = 0;
+            ti->start_frames_drop -= toskip;
+            }
+         else
+            {
+            toskip = ti->start_frames_drop;
+            pcmlength -= toskip;
+            ti->start_frames_drop = 0;
+            }
+         }
+      else
+         toskip = 0;
    
-   if (self->resample)
-      {
-      src_data->end_of_input = (pcm->length == 0);
-      src_data->input_frames = pcm->length;
-      src_data->data_in = dp = realloc(src_data->data_in, pcm->length * nchannels * sizeof (float));
-      src_data->output_frames = (int)(src_data->input_frames * src_data->src_ratio) + 2 + (512 * src_data->end_of_input);
-      src_data->data_out = realloc(src_data->data_out, src_data->output_frames * nchannels * sizeof (float));
-      for (nsamples = pcm->length; nsamples; nsamples--)
+      /* this ringbuffer prevents the last ti->end_frames_drop frames from being used */
+      /* resulting in gapless playback, provided the lame tag was present */
+      jack_ringbuffer_write(self->lrb, (char *)(left_ch + toskip), pcmlength << 2);
+      if (nchannels == 2)
+         jack_ringbuffer_write(self->rrb, (char *)(right_ch + toskip), pcmlength << 2);
+      else
+         jack_ringbuffer_write(self->rrb, (char *)(left_ch + toskip), pcmlength << 2);
+   
+      pcmlength = (jack_ringbuffer_read_space(self->lrb) >> 2) - ti->end_frames_drop;
+      if (pcmlength < 0)
+         pcmlength = 0;
+      if (!self->initial_data && pcmlength)
+         self->initial_data = TRUE;
+      
+      left_ch = lc = malloc(pcmlength << 2);
+      right_ch = rc = malloc(pcmlength << 2);
+   
+      if (!left_ch || !right_ch)
          {
-         *dp++ = scale(*left_ch++);
-         if (nchannels == 2)
-            *dp++ = scale(*right_ch++);
+         fprintf(stderr, "mp3decode_play: malloc failure\n");
+         exit(5);
          }
-      if (src_process(xlplayer->src_state, src_data))
-         {
-         fprintf(stderr, "mp3decode_play: error occured during resampling\n");
-         xlplayer->playmode = PM_EJECTING;
-         return;
-         }
-      xlplayer_demux_channel_data(xlplayer, src_data->data_out, src_data->output_frames_gen, pcm->channels);
+   
+      jack_ringbuffer_read(self->lrb, (char *)left_ch, pcmlength << 2);
+      jack_ringbuffer_read(self->rrb, (char *)right_ch, pcmlength << 2);
       }
    else
+      self->initial_data = TRUE;
+
+   if (self->initial_data)
       {
-      xlplayer->op_buffersize = (nsamples = pcm->length) * sizeof (float);
-      if (!(xlplayer->leftbuffer = lp = realloc(xlplayer->leftbuffer, xlplayer->op_buffersize)) && xlplayer->op_buffersize)
+      if (self->resample)
          {
-         fprintf(stderr, "mp3decode_play: malloc failure\n");
-         exit(5);
+         src_data->end_of_input = (pcmlength == 0);
+         src_data->input_frames = pcmlength;
+         src_data->data_in = dp = realloc(src_data->data_in, pcmlength * nchannels * sizeof (float));
+         src_data->output_frames = (int)(src_data->input_frames * src_data->src_ratio) + 2 + (512 * src_data->end_of_input);
+         src_data->data_out = realloc(src_data->data_out, src_data->output_frames * nchannels * sizeof (float));
+         for (nsamples = pcmlength; nsamples; nsamples--)
+            {
+            *dp++ = scale(*(lc++));
+            if (nchannels == 2)
+               *dp++ = scale(*(rc++));
+            }
+         if (src_process(xlplayer->src_state, src_data))
+            {
+            fprintf(stderr, "mp3decode_play: error occured during resampling\n");
+            xlplayer->playmode = PM_EJECTING;
+            if (ti->end_frames_drop)
+               {
+               free(left_ch);
+               free(right_ch);
+               }
+            return;
+            }
+         xlplayer_demux_channel_data(xlplayer, src_data->data_out, src_data->output_frames_gen, pcm->channels);
          }
-      if (!(xlplayer->rightbuffer = rp = realloc(xlplayer->rightbuffer, xlplayer->op_buffersize)) && xlplayer->op_buffersize)
+      else
          {
-         fprintf(stderr, "mp3decode_play: malloc failure\n");
-         exit(5);
+         xlplayer->op_buffersize = (nsamples = pcmlength) * sizeof (float);
+         if (!(xlplayer->leftbuffer = lp = realloc(xlplayer->leftbuffer, xlplayer->op_buffersize)) && xlplayer->op_buffersize)
+            {
+            fprintf(stderr, "mp3decode_play: malloc failure\n");
+            exit(5);
+            }
+         if (!(xlplayer->rightbuffer = rp = realloc(xlplayer->rightbuffer, xlplayer->op_buffersize)) && xlplayer->op_buffersize)
+            {
+            fprintf(stderr, "mp3decode_play: malloc failure\n");
+            exit(5);
+            }
+         while (nsamples--)
+            {
+            gain = xlplayer_get_next_gain(xlplayer);
+            *lp++ = gain * scale(*(lc++));
+            if (nchannels == 2)
+               *rp++ = gain * scale(*(rc++));
+            }
+         if (nchannels == 1)
+            memcpy(xlplayer->rightbuffer, xlplayer->leftbuffer, xlplayer->op_buffersize);
          }
-      while (nsamples--)
-         {
-         gain = xlplayer_get_next_gain(xlplayer);
-         *lp++ = gain * scale(*left_ch++);
-         if (nchannels == 2)
-            *rp++ = gain * scale(*right_ch++);
-         }
-      if (nchannels == 1)
-         memcpy(xlplayer->rightbuffer, xlplayer->leftbuffer, xlplayer->op_buffersize);
       }
-   xlplayer_write_channel_data(xlplayer);
+
+   if (ti->end_frames_drop)
+      {
+      free(left_ch);
+      free(right_ch);
+      }
+   if (self->initial_data)
+      xlplayer_write_channel_data(xlplayer);
    while((frame_code = mp3decode_get_frame(xlplayer)) == -1);
    if(frame_code)
       mad_synth_frame(&(self->synth), &(self->frame));
