@@ -370,8 +370,9 @@ static void xing_tag_read(struct mp3taginfo *ti, FILE *fp)
    int xing_offset, initial_offset, save_point = 0;
    int i, samples_per_frame, frame_length, padding;
    int bit_rate, sample_rate;
-   int flags;
+   int flags, b1, b2, b3;
    char xing_intro[4];
+   char lame_intro[4];
    static int side_info_table[2][2] = { { 17, 9 } , { 32, 17 } };
    static int bitrate_table[2][15] = {
       { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160 },
@@ -454,15 +455,31 @@ static void xing_tag_read(struct mp3taginfo *ti, FILE *fp)
 	    ti->have_toc = fread(ti->toc, 100, 1, fp);
 	    fprintf(stderr, "toc has been read\n");
 	    }
+	    
+	 if (flags & 0x8)
+	    be32bitread(fp);
+
+	 if (!fread(lame_intro, 4, 1, fp))
+	    goto no_tag;
+	    
+	 if (!memcmp(lame_intro, "LAME", 4))
+	    {
+	    fprintf(stderr, "lame tag found\n");
+	    fseek(fp, 17, SEEK_CUR);
+	    b1 = fgetc(fp);
+	    b2 = fgetc(fp);
+	    b3 = fgetc(fp);
+	    ti->start_frames_drop = ((b1 << 4) | (b2 >> 4)) + 528;
+	    ti->end_frames_drop = (((b2 & 0xF) << 8) | b3);
+	    fprintf(stderr, "frames to drop %d and %d\n", ti->start_frames_drop, ti->end_frames_drop);
+	    fseek(fp, 12, SEEK_CUR);
+	    }
+	 else
+	    fseek(fp, -4, SEEK_CUR);
 
 	 if (!frame_length)
-	    {
-            if (flags & 0x8)
-               save_point = ftell(fp) + 4;
-            else
-               save_point = ftell(fp);
-	    }
- 
+            save_point = ftell(fp);
+  
 	 if (!(ti->have_bytes))
 	    {
 	    fprintf(stderr, "deriving number of bytes manually\n");
