@@ -130,10 +130,14 @@ static void mp4decode_play(struct xlplayer *xlplayer)
       }
    if(!mp4ff_read_sample_v2(self->mp4handle, 0, self->mp4_sample_number++, self->mp4buffer))
       {
-      printf("read_sample_v2 returned zero indicating an error\n");
+      fprintf(stderr, "mp4decode_play: read_sample_v2 returned zero indicating an error\n");
+      goto finished;
+      }
+   if (!(data = faacDecDecode(self->faachandle, &frame_info, self->mp4buffer, buffersize)))
+      {
+      fprintf(stderr, "mp4decode_play: possible corrupt media file\n");
       goto finished;
       }
-   data = faacDecDecode(self->faachandle, &frame_info, self->mp4buffer, buffersize);
    self->mp4flbuf = realloc(self->mp4flbuf, frame_info.samples * sizeof (float));
    xlplayer_make_audio_to_float(xlplayer, self->mp4flbuf, data, frame_info.samples / frame_info.channels, 16, frame_info.channels); 
    if (self->faacresample_f)
