@@ -21,11 +21,97 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include "live_ogg_encoder.h"
 #include "sourceclient.h"
 #include "id3.h"
 
 #define TIMESTAMP_SIZ 23
 
+static void recorder_write_ogg_metaheader(struct recorder *self)
+   {
+   struct encoder *encoder = self->encoder_op->encoder;
+   struct loe_data *s = encoder->encoder_private;
+   vorbis_info vi;
+   vorbis_dsp_state vd;
+   vorbis_block vb;
+   vorbis_comment vc;
+   ogg_stream_state os;
+   ogg_page og;
+   ogg_packet op;
+   ogg_packet header_main;
+   ogg_packet header_comments;
+   ogg_packet header_codebooks;
+
+   void write_out(ogg_page *ogp)    /* output the ogg page */
+      {
+      fwrite(ogp->header, ogp->header_len, 1, self->fp);
+      fwrite(ogp->body, ogp->body_len, 1, self->fp);
+      if (ferror(self->fp))
+         {
+         fprintf(stderr, "recorder_write_ogg_metaheader: error writing the header\n");
+         }
+      }
+
+   void encode_silent_samples(int n_samples)
+      {
+      float **buffer;
+      int i;
+
+      /* generate a silent buffer */
+      buffer = vorbis_analysis_buffer(&vd, n_samples);
+      for (i = 0; i < vi.channels; i++)
+         memset(buffer[i], 0, n_samples * sizeof (float));
+      vorbis_analysis_wrote(&vd, n_samples);
+
+      /* encode it */
+      while (vorbis_analysis_blockout(&vd, &vb) == 1)
+         {
+         vorbis_analysis(&vb, NULL);
+         vorbis_bitrate_addblock(&vb);
+         while (vorbis_bitrate_flushpacket(&vd, &op))
+            {
+            ogg_stream_packetin(&os, &op);
+            while (ogg_stream_pageout(&os, &og))
+               {
+               write_out(&og);
+               if (ogg_page_eos(&og))
+                  break;
+               }
+            }
+         }
+      }
+
+   vorbis_info_init(&vi);
+   if (vorbis_encode_setup_managed(&vi, encoder->n_channels, encoder->target_samplerate, s->max_bitrate * 1000, encoder->bitrate * 1000, s->min_bitrate * 1000))
+      {
+      fprintf(stderr, "recorder_write_ogg_metaheader: mode initialisation failed\n");
+      vorbis_info_clear(&vi);
+      return;
+      }
+   vorbis_encode_setup_init(&vi);
+   vorbis_analysis_init(&vd, &vi);
+   vorbis_block_init(&vd, &vb);
+   ogg_stream_init(&os, self->initial_serial - 1);
+   vorbis_comment_init(&vc);
+
+                                    /* write vorbis header */
+   vorbis_analysis_headerout(&vd, &vc, &header_main, &header_comments, &header_codebooks);
+   ogg_stream_packetin(&os, &header_main);
+   ogg_stream_packetin(&os, &header_comments);
+   ogg_stream_packetin(&os, &header_codebooks);
+   while (ogg_stream_flush(&os, &og))
+      write_out(&og);
+
+   encode_silent_samples(1);        /* one sample is all we need */
+   encode_silent_samples(0);
+
+   ogg_stream_clear(&os);           /* cleanup */
+   vorbis_block_clear(&vb);
+   vorbis_dsp_clear(&vd);
+   vorbis_comment_clear(&vc);
+   vorbis_info_clear(&vi);
+   }
+
 static int recorder_write_id3_tag(struct recorder *self, FILE *fp)
    {
    struct metadata_item *mi;
@@ -355,7 +441,6 @@ static void *recorder_main(void *args)
    struct timespec ms10 = { 0, 10000000 };
    struct encoder_op_packet *packet;
       
-   fprintf(stderr, "recorder_main: got numeric id %d\n", self->numeric_id);
    while (!self->thread_terminate_f)
       {
       nanosleep(&ms10, NULL);
@@ -462,7 +547,6 @@ static void *recorder_main(void *args)
             fprintf(stderr, "recorder_main: unhandled record mode\n");
          }
       }
-   fprintf(stderr, "recorder_main: got the signal to terminate; acting on it\n");
    return NULL;
    }
 
@@ -529,6 +613,8 @@ int recorder_start(struct threads_info *ti, struct universal_vars *uv, void *oth
       return FAILED;
       }
    self->initial_serial = encoder_client_set_flush(self->encoder_op) + 1;
+   if (file_extension == ".ogg")
+      recorder_write_ogg_metaheader(self);
    fprintf(stderr, "streamer_main: connected to server - awaiting serial %d\n", self->initial_serial);
    if (self->pause_request == TRUE)
       self->record_mode = RM_PAUSED;
@@ -616,13 +702,11 @@ struct recorder *recorder_init(struct threads_info *ti, int numeric_id)
    self->threads_info = ti;
    self->numeric_id = numeric_id;  
    pthread_create(&self->thread_h, NULL, recorder_main, self);
-   fprintf(stderr, "recorder_init: created recorder %d\n", numeric_id);
    return self;
    }
 
 void recorder_destroy(struct recorder *self)
    {
-   fprintf(stderr, "recorder_destroy: taking out the recorder\n");
    self->thread_terminate_f = TRUE;
    pthread_join(self->thread_h, NULL);
    free(self);
