@@ -24,6 +24,7 @@
 #include <unistd.h>
 #include <jack/jack.h>
 #include <jack/ringbuffer.h>
+#include <signal.h>
 #include "sourceclient.h"
 #include "kvpparse.h"
 #include "live_ogg_encoder.h"
@@ -32,6 +33,8 @@ int keepalive = FALSE;
 int timeout_duration = 15;
 int debugmode = FALSE;
 
+static int threads_up;
+
 static int threads_init(struct threads_info *ti, struct universal_vars *uv, void *other)
    {
    struct threads_vars *tv = other;
@@ -77,6 +80,7 @@ static int threads_init(struct threads_info *ti, struct universal_vars *uv, void
       return FAILED;
       }
    fprintf(stderr, "started %d encoders, %d streamers, %d recorders\n", ti->n_encoders, ti->n_streamers, ti->n_recorders);
+   threads_up = TRUE;
    return SUCCEEDED;
    }
 
@@ -84,18 +88,21 @@ static void threads_shutdown(struct threads_info *ti)
    {
    int i;
    
-   fprintf(stderr, "threads_shutdown commencing\n");
-   watchdog_destroy(ti->watchdog);
-   audio_feed_destroy(ti->audio_feed);
-   for (i = 0; i < ti->n_recorders; i++)
-      recorder_destroy(ti->recorder[i]);
-   for (i = 0; i < ti->n_streamers; i++)
-      streamer_destroy(ti->streamer[i]);
-   for (i = 0; i < ti->n_encoders; i++)
-      encoder_destroy(ti->encoder[i]);
-   free(ti->recorder);
-   free(ti->streamer);
-   free(ti->encoder);
+   if (threads_up)
+      {
+      fprintf(stderr, "threads_shutdown commencing\n");
+      watchdog_destroy(ti->watchdog);
+      audio_feed_destroy(ti->audio_feed);
+      for (i = 0; i < ti->n_recorders; i++)
+         recorder_destroy(ti->recorder[i]);
+      for (i = 0; i < ti->n_streamers; i++)
+         streamer_destroy(ti->streamer[i]);
+      for (i = 0; i < ti->n_encoders; i++)
+         encoder_destroy(ti->encoder[i]);
+      free(ti->recorder);
+      free(ti->streamer);
+      free(ti->encoder);
+      }
    fprintf(stderr, "threads_shutdown finished\n");
    }
 
@@ -254,6 +261,7 @@ int main()
    setlocale(LC_ALL, "C");
 
    comms_send("succeeded");
+   srand(time(NULL));
    while (kvp_parse(kvpdict, stdin) || debugmode)
       {
       if (uv.command && command_parse(commandmap, &ti, &uv))
@@ -269,6 +277,8 @@ int main()
          uv.command = NULL;
          }
       }
+      
    threads_shutdown(&ti);
+   kvp_free_dict(kvpdict);
    return 0;
    }
