@@ -51,7 +51,6 @@ static void *streamer_main(void *args)
       return buffer;
       }
       
-   fprintf(stderr, "streamer_main: got numeric id %d\n", self->numeric_id);
    while (!self->thread_terminate_f)
       {
       nanosleep(&ms10, NULL);
@@ -171,7 +170,6 @@ static void *streamer_main(void *args)
             break;
          }
       }
-   fprintf(stderr, "streamer_main: got the signal to terminate; acting on it\n");
    return NULL;
    }
 
@@ -179,9 +177,10 @@ int streamer_make_report(struct streamer *self)
    {
    int buffer_fill_pc = 0;
    int new_connection = self->brand_new_connection; /* for thread safety */
-   
-   if (self->stream_mode == SM_CONNECTED && self->max_shout_queue)
-      buffer_fill_pc = (int)(shout_queuelen(self->shout) * 100 / self->max_shout_queue);
+   int max_shout_queue = self->max_shout_queue;
+
+   if (self->stream_mode == SM_CONNECTED && max_shout_queue)
+      buffer_fill_pc = (int)(shout_queuelen(self->shout) * 100 / max_shout_queue);
    fprintf(stdout, "idjcsc: streamer%dreport=%d:%d:%d\n", self->numeric_id, (int)self->stream_mode, buffer_fill_pc, new_connection);
    if (new_connection)
       self->brand_new_connection = FALSE;
@@ -194,6 +193,9 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
    struct streamer_vars *sv = other;
    struct streamer *self = ti->streamer[uv->tab];
    int protocol, data_format;
+   char channels[2];
+   char bitrate[4];
+   char samplerate[6];
 
    void sce(char *parameter)	/* stream connect error */
       {
@@ -230,6 +232,7 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
             return FAILED;
          }
       }
+      
    if (!strcmp(sv->server_type, "Shoutcast"))
       protocol = SHOUT_PROTOCOL_ICY;
    else if (!strcmp(sv->server_type, "Icecast 2"))
@@ -313,6 +316,27 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
       sce("make public");
       goto error;
       }
+      
+   snprintf(channels,   sizeof channels  , "%d",  self->encoder_op->encoder->n_channels);
+   snprintf(bitrate,    sizeof bitrate   , "%d",  self->encoder_op->encoder->bitrate);
+   snprintf(samplerate, sizeof samplerate, "%ld", self->encoder_op->encoder->samplerate);
+      
+   if (shout_set_audio_info(self->shout, SHOUT_AI_BITRATE, bitrate) != SHOUTERR_SUCCESS)
+      {
+      sce("set_audio_info bitrate");
+      goto error;
+      }
+   if (shout_set_audio_info(self->shout, SHOUT_AI_SAMPLERATE, samplerate) != SHOUTERR_SUCCESS)
+      {
+      sce("set_audio_info samplerate");
+      goto error;
+      }
+   if (shout_set_audio_info(self->shout, SHOUT_AI_CHANNELS, channels) != SHOUTERR_SUCCESS)
+      {
+      sce("set_audio_info channels");
+      goto error;
+      }
+      
    if (shout_set_nonblocking(self->shout, 1) != SHOUTERR_SUCCESS)
       {
       sce("non-blocking");
@@ -374,7 +398,6 @@ struct streamer *streamer_init(struct threads_info *ti, int numeric_id)
    self->threads_info = ti;
    self->numeric_id = numeric_id;
    pthread_create(&self->thread_h, NULL, streamer_main, self);
-   fprintf(stderr, "streamer_init: created streamer %d\n", numeric_id);
    return self;
    }
 
@@ -386,6 +409,5 @@ void streamer_destroy(struct streamer *self)
    pthread_once(&once_control, shout_shutdown);
    self->thread_terminate_f = TRUE;
    pthread_join(self->thread_h, &thread_ret);
-   fprintf(stderr, "streamer_destroy: taking out the streamer\n");
    free(self);
    }
