@@ -300,7 +300,7 @@ static void *xlplayer_main(struct xlplayer *self)
 #endif
 #ifdef HAVE_AVCODEC
 #ifdef HAVE_AVFORMAT
-                    || ((!strcmp(extension, "wma") || !strcmp(extension, "avi")) && avcodecdecode_reg(self))
+                    || ((!strcmp(extension, "m4a") || !strcmp(extension, "mp4") || !strcmp(extension, "wma") || !strcmp(extension, "avi")) && avcodecdecode_reg(self))
 #endif /* HAVE_AVFORMAT */
 #endif /* HAVE_AVCODEC */
 #ifdef HAVE_MAD
@@ -537,6 +537,12 @@ struct xlplayer *xlplayer_create(int samplerate, size_t rbsize, char *playername
    self->up = FALSE;
    self->pbspeed = 1.0F;
    self->pbs_exchange = 0;
+   self->dynamic_metadata.data_type = DM_NONE_NEW;
+   self->dynamic_metadata.artist = NULL;
+   self->dynamic_metadata.title = NULL;
+   self->dynamic_metadata.artist_title = NULL;
+   self->dynamic_metadata.current_audio_context = 0;
+   self->dynamic_metadata.rbdelay = 0;
    pthread_create(&self->thread, NULL, (void *(*)(void *)) xlplayer_main, self);
    while (self->up == FALSE)
       usleep(10000);
@@ -859,6 +865,7 @@ void xlplayer_destroy_fadein_lookup()
 void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t type, char *artist, char *title, char *artist_title, int usedelay)
    {
    struct xlp_dynamic_metadata *dm = &(xlplayer->dynamic_metadata);
+   int min;
    
    pthread_mutex_lock(&(dm->meta_mutex));
    dm->data_type = type;
@@ -872,6 +879,7 @@ void xlplayer_set_dynamic_metadata(struct xlplayer *xlplayer, enum metadata_t ty
    dm->title = strdup(title);
    dm->artist_title = strdup(artist_title);
    dm->current_audio_context = xlplayer->current_audio_context;
-   dm->rbdelay = xlplayer->rbdelay * usedelay;
+   min = (xlplayer->rbdelay < xlplayer->play_progress_ms) ? xlplayer->rbdelay : xlplayer->play_progress_ms;
+   dm->rbdelay = min * usedelay;
    pthread_mutex_unlock(&(dm->meta_mutex));
    }
